//
//  ISPDataMinerDisks.m
//  iStatMenusDisks
//
//  Created by iSlayer on 17/04/07.
//  Copyright 2007 iSlayer & Distorted Vista. All rights reserved.
//

#import "ISPDataMinerDisks.h"

@implementation ISPDataMinerDisks

- (id)init {
	icon_directory = [[NSString alloc] initWithString:[@"~/Library/Caches/iStatPro/" stringByExpandingTildeInPath]];
	self = [super init];

	BOOL isDir;
	
	BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:icon_directory isDirectory:&isDir];
    if (!exists){
		[[NSFileManager defaultManager] createDirectoryAtPath:icon_directory attributes:nil];
	} else {
		// If the directory exists already then delete it to force icons to be updated - need to remove this and do a proper NSImage comparison elsewhere
		[[NSFileManager defaultManager] removeFileAtPath:icon_directory handler:nil];
		[[NSFileManager defaultManager] createDirectoryAtPath:icon_directory attributes:nil];
	}

	driveTypes = [[NSMutableDictionary alloc] init];
	[self findDrives];
	[self getDataSet];
	
	return self;
}

- (void) dealloc {
	[driveTypes removeAllObjects];
	[driveTypes release], driveTypes = nil;
	[icon_directory release], icon_directory = nil;
	[super dealloc];
}

- (NSArray *)getDataSet {
	NSMutableArray *theData = [[NSMutableArray alloc] init];
    NSString *path;
	BOOL first = YES;
	int i = 0;
    NSEnumerator *mountedPathsEnumerator = [[[NSWorkspace  sharedWorkspace] mountedLocalVolumePaths] objectEnumerator];
    while (path = [mountedPathsEnumerator nextObject] ) {
		struct statfs buffer;
        int returnnewCode = statfs([path fileSystemRepresentation],&buffer);
        if ( returnnewCode == 0 ) {
			NSRange start = [path rangeOfString:@"/Volumes/"];
			if(first == NO && start.length == 0){
				// If the disk is not mounted to /Volumes then we are not interested in it 
				continue;
			}
	
			// This is just a simple check to see if we are looking at the boot drive. Boot disks do not show up as being mounted in /Volumes so we ignore the first volume
			if(first)
				first = NO;

			FSRef pathRef;
			FSPathMakeRef((const UInt8 *)[path fileSystemRepresentation], &pathRef, NULL);

			FSCatalogInfo catInfo;
			FSGetCatalogInfo(&pathRef, kFSCatInfoVolume, &catInfo, NULL, NULL, NULL);

			FSVolumeInfo info;
			FSGetVolumeInfo (catInfo.volume, 0, NULL, kFSVolInfoSizes, &info, NULL, NULL);

			// Get the figures
			NSNumber* driveSize = [NSNumber numberWithUnsignedLongLong:info.totalBytes / 1024];
			NSNumber* driveFree = [NSNumber numberWithUnsignedLongLong:info.freeBytes / 1024];
			NSNumber* driveUsed = [NSNumber numberWithInt:(info.totalBytes - info.freeBytes ) / 1024];
				
			// if size == NSNotFound then something has gone wrong. This always happens with iDisks when there is a local copy.
			if([driveSize intValue] == NSNotFound)
				continue;
			
			NSNumber* drivePercentage = [NSNumber numberWithInt:([driveUsed floatValue] / [driveSize floatValue]) * 100];
			
			NSString *name = [[NSString stringWithFormat:@"%s",buffer.f_mntfromname] lastPathComponent];

			if([name hasPrefix:@"disk"] && [name length] > 4){
				NSString *newName = [name substringFromIndex:4];
				NSRange paritionLocation = [newName rangeOfString:@"s"];
				if(paritionLocation.length != 0){
					name = [NSString stringWithFormat:@"disk%@",[newName substringToIndex: paritionLocation.location]];
				}
			}

			if([driveTypes objectForKey:name] != NULL && ![path isEqualToString:@"/Volumes/iDisk"]){
				if([[driveTypes objectForKey:name] isEqualToString:@"Virtual Interface"] || [[driveTypes objectForKey:name] isEqualToString:@"CD"] || [[driveTypes objectForKey:name] isEqualToString:@"DVD"])
					continue;
			}

			NSString *iconFilePath = [NSString stringWithFormat:@"%@/%@.tiff",icon_directory,[[NSFileManager defaultManager] displayNameAtPath:path]];
			if([[NSFileManager defaultManager] fileExistsAtPath:iconFilePath] == NO) {
				NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:path];       
				NSArray *reps = [icon representations];
				if([reps count] > 0){
					int x;
					float smallestSize = FLT_MAX;
					int index = 0;
					
					for(x=0;x<[reps count];x++){
						NSSize size = [[reps objectAtIndex:x] size];
						if(size.width < smallestSize && size.width > 16){
							smallestSize = size.width;
							index = x;
						}
					}
					NSImageRep *rep = [reps objectAtIndex:index];
					NSImage *smallIcon = [[NSImage alloc] initWithSize:[rep size]];
					[smallIcon addRepresentation:rep];
					
					NSData *tiffRep = [smallIcon TIFFRepresentation];
					[tiffRep writeToFile:iconFilePath atomically:YES];
					[smallIcon release];
				} else
					iconFilePath = @"";
			}
			
			[theData addObject:[NSArray arrayWithObjects:[[NSFileManager defaultManager] displayNameAtPath:path],drivePercentage,[self formattedSize:driveUsed],[self formattedSize:driveFree],path, iconFilePath, nil]];
			i++;
        }
	}

	return [theData autorelease];
}

- (void)findDrives {
	[driveTypes removeAllObjects];
    IONotificationPortRef port;

    port = IONotificationPortCreate(kIOMasterPortDefault);

    if(port){
        CFMutableDictionaryRef properties;
        properties = IOServiceMatching(kIOMediaClass);
        if(properties){
            io_iterator_t notification;
			CFDictionaryAddValue(properties, CFSTR(kIOMediaWholeKey), kCFBooleanTrue);
            notification = 0;
			IOServiceAddMatchingNotification(port, kIOMatchedNotification, properties, NULL, NULL, &notification);
            if(notification) {
                [self callback:notification];
                IOObjectRelease(notification);
            }
        }
		
        IONotificationPortDestroy(port);
    }
}

- (void)callback:(io_iterator_t)notification {
    io_object_t media;

    while((media = IOIteratorNext(notification))) {
		NSString* bsdName = NULL;
        CFDictionaryRef properties = NULL;

		IORegistryEntryCreateCFProperties( media, ( CFMutableDictionaryRef * ) &properties, kCFAllocatorDefault, 0 );

        if(properties){
 			NSDictionary* disk = [NSDictionary dictionaryWithDictionary:(NSDictionary* )properties];
			bsdName = [disk objectForKey:@"BSD Name"];
			CFRelease( properties );
        } else
			continue;
		
		if(IOObjectConformsTo(media, kIOCDMediaClass)){
			[driveTypes setValue:@"CD" forKey:bsdName];
			IOObjectRelease( media );
			continue;
		}
		if(IOObjectConformsTo( media, kIODVDMediaClass ) ){
			[driveTypes setValue:@"DVD" forKey:bsdName];
			IOObjectRelease( media );
			continue;
		}
		
		properties = IORegistryEntrySearchCFProperty( media, kIOServicePlane, CFSTR( kIOPropertyProtocolCharacteristicsKey ), kCFAllocatorDefault, kIORegistryIterateParents | kIORegistryIterateRecursively );
        if(properties){
 			NSDictionary* disk = [NSDictionary dictionaryWithDictionary:(NSDictionary* )properties];
			[driveTypes setValue:[disk objectForKey:@"Physical Interconnect"] forKey:bsdName];
            CFRelease( properties );
        }

        IOObjectRelease( media );
    }
}

- (NSString *)formattedSize:(NSNumber *)input {
	unsigned long long value = [input unsignedLongLongValue];

	if(value < 1024 || value == LONG_LONG_MAX) // KB
		return @"0<span class='size'>KB</span>";

	if(value < 1048576){ // MB
		return [NSString stringWithFormat:@"%llu<span class='size'>MB</span>",value / 1024];
	}
	
	if(value < 1073741824){ // GB
		float valueF = value / 1048576.0;
		if(valueF > 99)
			return [NSString stringWithFormat:@"%.1f<span class='size'>GB</span>", valueF];
		else
			return [NSString stringWithFormat:@"%.2f<span class='size'>GB</span>", valueF];
	}

	float valueF = value / 1073741824.0;
	if(valueF > 99)
		return [NSString stringWithFormat:@"%.1f<span class='size'>TB</span>",valueF];
	else	
		return [NSString stringWithFormat:@"%.2f<span class='size'>TB</span>",valueF];
}

@end
